/* Copyright 2021 - 2022 R. Thomas
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef LIEF_THIRD_PARTY_SPAN_H_
#define LIEF_THIRD_PARTY_SPAN_H_
#include "LIEF/config.h"

#ifndef LIEF_EXTERNAL_SPAN
#include <LIEF/third-party/internal/span.hpp>
#else
#include <tcb/span.hpp>
#endif

#endif
